﻿#region LICENSE
//
// Copyright (c) Microncode.com
// 
// This source is subject to the Microsoft Public License. 
// See https://opensource.org/licenses/ms-pl.html.
// All other rights reserved. 
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
//
#endregion

namespace CSFreeDBExample
{
    using System;
    using System.Windows.Forms;

    /// <summary>
    /// The main example of using the CSFreeDB library.
    /// </summary>
    public partial class frmFreeDB : Form
    {
        private CSFreeDB.FreeDB freeDB1;

        /// <summary>
        /// Constructor.
        /// </summary>
        public frmFreeDB()
        {
            InitializeComponent();

            this.freeDB1 = new CSFreeDB.FreeDB();

            this.freeDB1.FreeDBStatus += new CSFreeDBLib.FreeDB.FreeDBStatusEventHandler(this.freeDB1_FreeDBStatus);
            this.freeDB1.FreeDBError += new CSFreeDBLib.FreeDB.FreeDBErrorEventHandler(this.freeDB1_FreeDBError);
            this.freeDB1.FreeDBTracks += new CSFreeDBLib.FreeDB.FreeDBTracksEventHandler(this.freeDB1_FreeDBTracks);
            this.freeDB1.FreeDBAlbum += new CSFreeDBLib.FreeDB.FreeDBAlbumEventHandler(this.freeDB1_FreeDBAlbum);
            this.freeDB1.FreeDBDone += new CSFreeDBLib.FreeDB.FreeDBDoneEventHandler(this.freeDB1_FreeDBDone);
        }

        /// <summary>
        /// Form load.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void frmFreeDB_Load(object sender, EventArgs e)
        {
            freeDB1.UserName = "Your email";
            freeDB1.UserKey = "Your registration key";


            cboDevice.DataSource = freeDB1.DrivesInfo;
        }

        /// <summary>
        /// Open the door of the CD drive.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpen_Click(object sender, EventArgs e)
        {
            lstTracks.Items.Clear();
            freeDB1.CDEject(cboDevice.SelectedIndex);
        }

        /// <summary>
        /// Close the door of the CD drive.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            lstTracks.Items.Clear();
            freeDB1.CDClose(cboDevice.SelectedIndex);
        }

        /// <summary>
        /// Refresh the CD tracks.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRefresh_Click(object sender, EventArgs e)
        {
            LoadCDTracks();
        }

        /// <summary>
        /// Load the CD tracks to the list view.
        /// </summary>
        /// <returns></returns>
        int LoadCDTracks()
        {
            if (freeDB1.CDIsReady(cboDevice.SelectedIndex) == false)
            {
                MessageBox.Show("The CD is not ready. Please insert an Audio CD to the drive.");
                return -1;
            }
            else
            {

                lstTracks.Items.Clear();

                string[] arr = new string[6];
                ListViewItem itm;

                foreach (CSFreeDB.FreeDB.TrackInfo track in freeDB1.GetTracks(cboDevice.SelectedIndex))
                {
                    arr[0] = track.TrackNumber.ToString();
                    arr[1] = "Track" + track.TrackNumber.ToString("D2") + ".cda";
                    arr[2] = track.TrackLength.ToString("hh\\:mm\\:ss");
                    arr[3] = track.TrackBytes.ToString();
                    arr[4] = track.TrackType;
                    arr[5] = track.TrackFile;

                    itm = new ListViewItem(arr);

                    lstTracks.Items.Add(itm);
                }

                if (lstTracks.Items.Count > 0)
                {
                    lstTracks.Items[0].Selected = true;

                }

                return 0;
            }
        }

        /// <summary>
        /// Load the track from the selected audio CD.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cboDevice_SelectedIndexChanged(object sender, EventArgs e)
        {
            LoadCDTracks();
        }

        /// <summary>
        /// Load the FreeDB information.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdLoadFreeDB_Click(object sender, EventArgs e)
        {

            if (freeDB1.CDIsReady(cboDevice.SelectedIndex) == false)
            {
                MessageBox.Show("The CD is not ready. Please insert an audio cd to the drive.");
                return;
            }
            else
            {
                //Call the FreeDB and receive the data in the events:
                freeDB1.GetFreeDBInfo(cboDevice.SelectedIndex);
            }
        }

        /// <summary>
        /// Load the FreeDB information in Async mode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdLoadFreeDBAsync_Click(object sender, EventArgs e)
        {
            int cntr = 0;

            // Tracks names:
            foreach (string trckFreeDB in freeDB1.GetFreeDBInfoAsync(cboDevice.SelectedIndex).FreeDBTracksInfo)
            {
                lstTracks.Items[cntr].SubItems[1].Text = trckFreeDB.ToString();
                cntr = cntr + 1;
            }

            //Album / Artist name:
            if (cntr > 0)
            {
                lblArtist.Text = "Artist: " + freeDB1.GetFreeDBInfoAsync(cboDevice.SelectedIndex).Artist;
                lblAlbum.Text = "Album: " + freeDB1.GetFreeDBInfoAsync(cboDevice.SelectedIndex).AlbumName;
                lblAlbumDiscID.Text = "DiscID: " + freeDB1.GetFreeDBInfoAsync(cboDevice.SelectedIndex).DiscID;
            }
        }

        //Events:
        
        /// <summary>
        /// When done.
        /// </summary>
        /// <param name="sender"></param>
        private void freeDB1_FreeDBDone(object sender)
        {
            toolStripStatusLabel1.Text = "Done.";
        }

        /// <summary>
        /// On error.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void freeDB1_FreeDBError(object sender, CSFreeDB.Core.MessageArgs e)
        {
            MessageBox.Show(e.String + "(" + e.Number + ")");

            toolStripStatusLabel1.Text = e.String + "(" + e.Number + ")";
        }

        /// <summary>
        /// When the status has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void freeDB1_FreeDBStatus(object sender, CSFreeDB.Core.MessageArgs e)
        {
            Console.WriteLine(e.String);
            toolStripStatusLabel1.Text = e.String;
        }

        /// <summary>
        /// The information about the album.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void freeDB1_FreeDBAlbum(object sender, CSFreeDBLib.FreeDB.AlbumInfo e)
        {
            lblArtist.Text = "Artist: " + e.AlbumArtist;
            lblAlbum.Text = "Album: " + e.AlbumName;
            lblAlbumDiscID.Text = "DiscID: " + e.AlbumDiscID;

        }

        /// <summary>
        /// The information about the tracks.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void freeDB1_FreeDBTracks(object sender, CSFreeDBLib.FreeDB.TrackInfo e)
        {
            

            Console.WriteLine(e.TrackName);

            lstTracks.Items[e.TrackNumber - 1].SubItems[1].Text = e.TrackName;
        }

        /// <summary>
        /// Order a license.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdOrderLicense_Click(object sender, EventArgs e)
        {
            /// In order to get your own UserName and RegKey and to distribute this library
            /// with your own projects for commercial or any other purpose, please order a license at:
            /// https://www.microncode.com/developers/cs-freedb/?cmd=order

            System.Diagnostics.Process.Start("https://www.microncode.com/developers/cs-freedb/?cmd=order");
        }
    }
}
